"""Version for ansys-dpf-core"""
# major, minor, patch
version_info = 0, 3, 1

# Nice string for the version
__version__ = ".".join(map(str, version_info))
__ansys_version__ = "212"
min_server_version = "2.0"
